;****************************************************************************
;                                                                           *
;   Filename:		00069R2.asm                                             *
;   Date:          	23SEP05                                                 *
;   File Version:  	v2.0                                                    *
;																			*
;	Software License Agreement												*
;																			*
;	The software supplied herewith by Microchip Technology Incorporated		*
;	(the "Company") for its PICmicro Microcontroller is intended and		*
;	supplied to you, the Company's customer, for use solely and				*
;	exclusively on Microchip PICmicro Microcontroller products. The			*
;	software is owned by the Company and/or its supplier, and is			*	
;	protected under applicable copyright laws. All rights are reserved.		*
;	Any use in violation of the foregoing restrictions may subject the		*
;	user to criminal sanctions under applicable laws, as well as to			*
;	civil liability for the breach of the terms and conditions of this		*	
;	license.																*		
;																			*	
;	THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,		*	
;	WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED		*	
;	TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A				*	
;	PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,		*	
;	IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR				*	
;	CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.						*	
;																			*
;                                                                           *
;	Author:        Scott Dearborn                                           *
;	Company:       Microchip Technology, Inc.                               *
;                                                                           * 
;****************************************************************************
;                                                                           *
;   Files required:         P12F683.INC                                     *
;                           12F683.LKR										*
;                                                                           *
;****************************************************************************

	list      p=12F683            ; list directive to define processor
	#include <p12F683.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

;****************************************************************************
;Configuration bits															
; The __CONFIG directive defines configuration data within the .ASM file.	
; The PIC12F683 Data Sheet explains the functions of the configuration bits.

	__CONFIG   0x33E4 


;****************************************************************************
;Constant definitions

;BC_STATUS bits
TEMP_MONITOR		EQU	0x00
OVER_VOLTAGE		EQU	0x01
PRECONDITION		EQU	0x02
CURRENT_RAMP		EQU	0x03
REGULATE_CHARGE		EQU	0x04

;Battery Voltages
VBAT43H				EQU	0xDF		;overvoltage threshold
VBAT42H				EQU	0xD6		;battery regulation 
VBAT405H			EQU	0xCF		;battery recharge threshold
VBAT40H				EQU	0xCD		;battery recharge threshold
VBAT30H				EQU	0x99		;preconditioning threshold
VBAT28H				EQU	0x90		;preconditioning threshold
VBAT25H				EQU	0x80		;fault threshold
VBAT20H				EQU	0x66		;connection threshold

;Battery Temperatures
TEMP_HIGH			EQU	0xC5		;0 degrees Celcius
TEMP_HIGH_HYS		EQU	0xC0
TEMP_LOW			EQU	0x50		;50 degrees Celcius
TEMP_LOW_HYS		EQU	0x55

;Charge Currents
;PI_PWM1				EQU	0xEB		;150mA Preconditioning Current
PI_PWM1				EQU	0xE6		;200mA Preconditioning Current
;FI_PWM1				EQU	0x09		;2000mA Fast Charge Current
FI_PWM1				EQU	0x42		;1500mA Fast Charge Current
;FI_PWM1				EQU	0x68		;1200mA Fast Charge Current
TI_PWM1				EQU	0xF1		;100mA Termination Current
I_PWM2				EQU	0x1B
I_PWM3				EQU	0x1D
PWM50				EQU	0xFA
FAST_RAMP_STEP		EQU	0x08
;FAST_RAMP_STEP		EQU	0x0F

;Timer
PRECOND_TIME		EQU	0x16		;22 minutes
ELAPSED_TIME		EQU	0xB4		;180 minutes
PWR_UP_DELAY		EQU	0x08		;2 seconds
OV_DELAY			EQU	0x28		;10 seconds
RECHARGE_DELAY		EQU	0x28		;10 seconds

;LOOPS_IN_MINUTE		EQU	0xE4		;change with TIME_IN_LOOPS
;TIME_IN_LOOPS		EQU	0x01		;change with LOOPS_IN_MINUTE

;LOOPS_IN_MINUTE		EQU	0x72		;change with TIME_IN_LOOPS
;TIME_IN_LOOPS		EQU	0x02		;change with LOOPS_IN_MINUTE

;LOOPS_IN_MINUTE		EQU	0x4C		;change with TIME_IN_LOOPS
;TIME_IN_LOOPS		EQU	0x03		;change with LOOPS_IN_MINUTE

LOOPS_IN_MINUTE		EQU	0x39		;change with TIME_IN_LOOPS
TIME_IN_LOOPS		EQU	0x04		;change with LOOPS_IN_MINUTE
PERSISTANCE_COUNT	EQU	0x0A

;GPIO
LED					EQU	0x04
PWM					EQU	0x05 

;****************************************************************************
;Variable definitions

INT_VAR				UDATA_SHR	0x70   
BC_STATUS			RES	1
VBATH				RES	1
TEMPH				RES	1
PWM1				RES	1
PWM2				RES	1
PWM3				RES	1
DELAY_COUNT			RES	1
LOOP_COUNT			RES	1
MINUTE_COUNT		RES	1
TIMEOUT				RES	1
VBAT42CAL			RES	1
PERSISTANCE			RES	1

;****************************************************************************
RESET_VECTOR	CODE	0x000		; processor reset vector
		goto    MAIN              	; go to beginning of program


INT_VECTOR	CODE	0x004			; interrupt vector location
		bsf		CCPR1L,7
		bsf		BC_STATUS,OVER_VOLTAGE
		movf	CMCON0,F
		clrf	PIR1

		retfie                    ; return from interrupt


MAIN
;****************************************************************************
;Initialize oscillator
		movlw	0x71				;set internal oscillator to 8MHz
		bsf		STATUS,RP0
		movwf	OSCCON
CLOCK_STABLE?
		btfss	OSCCON,HTS			;frequency stable?
		goto	CLOCK_STABLE?		;no, wait

;****************************************************************************
;Initialize ports
		movlw	0x0B				 
		movwf	TRISIO				;set CCP1, GP4, GP5 to output
		movlw	0x04				;set CCP1 to a logic HIGH
		bcf		STATUS,RP0
		movwf	GPIO

;****************************************************************************
;Initialize ADC
		movlw	0x05
		movwf	ADCON0
		movlw	0x53
		bsf		STATUS,RP0
		movwf	ANSEL

;****************************************************************************
;Initialize comparator
		movlw	0x04
		bcf		STATUS,RP0
		movwf	CMCON0
		movlw	0xAD
		bsf		STATUS,RP0
		movwf	VRCON

;****************************************************************************
;Initialize PWM
		movlw	0x03
		movwf	PR2
		movlw	0x80
		bcf		STATUS,RP0
		movwf	CCPR1L
		movlw	0x3C
		movwf	CCP1CON

;****************************************************************************
;Initialize Timers
		clrf	TMR1H
		clrf	TMR1L
		bcf		PIR1,TMR1IF
		movlw	0x31
		movwf	T1CON

;****************************************************************************
;Initialize Variables
		movlw	0x70
		movwf	FSR
NEXT	clrf	INDF
		incf	FSR,F
		btfss	FSR,7
		goto	NEXT

;		bsf		BC_STATUS,TEMP_MONITOR	;REMOVE THIS LINE OF CODE IF TEMPERATURE
										;MONITORING IS NOT USED

;****************************************************************************
;POWER_UP_DELAY
		movlw	PWR_UP_DELAY
		movwf	DELAY_COUNT

DELAY_LOOP
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		decfsz	DELAY_COUNT,F
		goto	DELAY_LOOP

;****************************************************************************
;INITIAL POWER_UP_CALIBRATION
		clrf	INTCON
		bsf		STATUS,RP0
		clrf	EEADR
		bsf		EECON1,RD
		btfsc	EEDAT,0
		goto	READ_CALIBRATION_DATA
CALIBRATE
		bcf		STATUS,RP0
		bcf		GPIO,LED
		bsf		ADCON0,GO
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bsf		STATUS,RP0
		rlf		ADRESL,F
		bcf		STATUS,RP0
		rlf		ADRESH,W
		movwf	VBATH
		bsf		ADCON0,GO
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bsf		STATUS,RP0
		rlf		ADRESL,F
		bcf		STATUS,RP0
		rlf		ADRESH,W
		addwf	VBATH,F
		rrf		VBATH,F
		movlw	VBAT40H
		subwf	VBATH,W
		btfsc	STATUS,C
		goto	WRITE_CALIBRATION_DATA
		movlw	VBAT42H
		movwf	VBATH
WRITE_CALIBRATION_DATA
		bcf		PIR1,EEIF
		bsf		STATUS,RP0
		clrf	EEADR
		movlw	0xFF
		movwf	EEDAT
		bsf		EECON1,WREN
		movlw	0x55
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf		EECON1,WR
		bcf		EECON1,WREN
		bcf		STATUS,RP0
		btfss	PIR1,EEIF
		goto	$-1
		bcf		PIR1,EEIF
		
		bsf		STATUS,RP0
		movlw	0x01
		movwf	EEADR
		movf	VBATH,W
		movwf	EEDAT
		bsf		EECON1,WREN
		movlw	0x55
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf		EECON1,WR
		bcf		EECON1,WREN
		bcf		STATUS,RP0
		btfss	PIR1,EEIF
		goto	$-1
		bcf		PIR1,EEIF

CALIBRATION_COMPLETE
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bsf		GPIO,LED
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bcf		GPIO,LED
		goto	CALIBRATION_COMPLETE

READ_CALIBRATION_DATA		
		movlw	0x01
		movwf	EEADR
		bsf		EECON1,RD
		movf	EEDAT,W
		movwf	VBAT42CAL
		bcf		STATUS,RP0
		clrf	INTCON

;****************************************************************************
;CHARGE_CYCLE_START
CHARGE_CYCLE_START
		bcf		GPIO,LED
		bsf		ADCON0,GO
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		movf	ADRESH,W
		movwf	VBATH
		rlf		VBATH,F
		movf	VBATH,W
		sublw	VBAT20H
		btfsc	STATUS,C
		goto	CHARGE_CYCLE_START

		movlw	TIME_IN_LOOPS
		movwf	LOOP_COUNT
		movlw	LOOPS_IN_MINUTE
		movwf	MINUTE_COUNT
		movlw	PRECOND_TIME
		movwf	TIMEOUT
		movlw	PERSISTANCE_COUNT
		movwf	PERSISTANCE
		movlw	PI_PWM1
		movwf	PWM1
		movlw	I_PWM2
		movwf	PWM2
		movlw	I_PWM3
		movwf	PWM3
		clrf	CCPR1L
		bsf		T2CON,TMR2ON
		bsf		GPIO,LED
		bsf		BC_STATUS,PRECONDITION

INITIALIZE_OV_INTERRUPT
		movf	CMCON0,F
		clrf	PIR1
		bsf		STATUS,RP0
		movlw	0x08
		movwf	PIE1
		bcf		STATUS,RP0
		movlw	0xC0
		movwf	INTCON

		movlw	HIGH(PWM_TABLE)
		movwf	PCLATH
						
VREF_PWM_LOOP
		bsf		ADCON0,GO
		movf	PWM1,W
		bsf		GPIO,PWM
		call	PWM_TABLE
		bcf		GPIO,PWM
		sublw	PWM50
		call	PWM_TABLE
		bcf		ADCON0,CHS0
		bsf		STATUS,RP0
		rlf		ADRESL,W
		bcf		STATUS,RP0
		rlf		ADRESH,W
		addwf	VBATH,F
		rrf		VBATH,F
		movf	PWM2,W
		call	PWM_TABLE
		bsf		ADCON0,GO
		movf	PWM1,W
		bsf		GPIO,PWM
		call	PWM_TABLE
		bcf		GPIO,PWM
		sublw	PWM50
		call	PWM_TABLE
		bsf		ADCON0,CHS0
		movf	ADRESH,W
		addwf	TEMPH,F
		rrf		TEMPH,F
		movf	PWM3,W
		call	PWM_TABLE
		btfsc	BC_STATUS,OVER_VOLTAGE
		goto	BATTERY_REMOVED			;battery disconnected
		btfss	PIR1,TMR1IF
		goto	VREF_PWM_LOOP

		bcf		PIR1,TMR1IF
		decfsz	LOOP_COUNT,F
		goto	VREF_PWM_LOOP
		movlw	TIME_IN_LOOPS
		movwf	LOOP_COUNT
		decfsz	MINUTE_COUNT,F
		goto	CHECK_TEMPERATURE
		movlw	LOOPS_IN_MINUTE
		movwf	MINUTE_COUNT
		decfsz	TIMEOUT,F
		goto	CHECK_TEMPERATURE
		goto	TIMEOUT_FAULT			;latched fault - safety timer

CHECK_TEMPERATURE
		btfss	BC_STATUS,TEMP_MONITOR
		goto	CHECK_VBAT
		movf	TEMPH,W
		sublw	TEMP_HIGH
		btfss	STATUS,C
		goto	TEMPERATURE_FAULT
		movf	TEMPH,W
		sublw	TEMP_LOW
		btfsc	STATUS,C
		goto	TEMPERATURE_FAULT

CHECK_VBAT
		btfsc	BC_STATUS,CURRENT_RAMP
		goto	CHECK_VBAT_RAMP
		btfsc	BC_STATUS,REGULATE_CHARGE
		goto	CHECK_VBAT_REG

CHECK_VBAT_PRECOND
		movf	VBATH,W
		sublw	VBAT30H
		btfsc	STATUS,C
		goto	VREF_PWM_LOOP

FAST_CHARGE_RAMP
		movlw	ELAPSED_TIME
		movwf	TIMEOUT
		bcf		BC_STATUS,PRECONDITION
		bsf		BC_STATUS,CURRENT_RAMP
		goto	VREF_PWM_LOOP

CHECK_VBAT_RAMP
		movf	VBATH,W
		sublw	VBAT25H
		btfsc	STATUS,C
		goto	RETURN_TO_PRECONDITION		
		movf	VBATH,W
		subwf	VBAT42CAL,W
		btfss	STATUS,C
		goto	REGULATE_CURRENT
		movlw	FAST_RAMP_STEP
		subwf	PWM1,W
		sublw	FI_PWM1
		btfsc	STATUS,C
		goto	REGULATE_CURRENT
		movlw	FAST_RAMP_STEP
		subwf	PWM1,F
		goto	VREF_PWM_LOOP

REGULATE_CURRENT
		bcf		BC_STATUS,CURRENT_RAMP
		bsf		BC_STATUS,REGULATE_CHARGE
		goto	VREF_PWM_LOOP

CHECK_VBAT_REG
		movf	VBATH,W
		sublw	VBAT25H
		btfsc	STATUS,C
		goto	RETURN_TO_PRECONDITION		
		movf	VBATH,W
		subwf	VBAT42CAL,W
		btfsc	STATUS,Z
		goto	NO_CHANGE
		btfss	STATUS,C
		goto	DECREMENT_CURRENT

INCREMENT_CURRENT
		decfsz	PERSISTANCE,F
		goto	VREF_PWM_LOOP
		movlw	FI_PWM1
		subwf	PWM1,W
		btfsc	STATUS,C
		decf	PWM1,F
NO_CHANGE
		movlw	PERSISTANCE_COUNT
		movwf	PERSISTANCE
		goto	VREF_PWM_LOOP

DECREMENT_CURRENT
		incf	PWM1,F
		movf	PWM1,W
		sublw	TI_PWM1
		btfsc	STATUS,C
		goto	VREF_PWM_LOOP

CHARGE_COMPLETE_LOOP
		bsf		CCPR1L,7
		bcf		GPIO,LED
		movlw	RECHARGE_DELAY
		movwf	DELAY_COUNT
		movlw	0x01
		andwf	BC_STATUS,F
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bsf		ADCON0,GO
		btfsc	ADCON0,GO
		goto	$-1
		rlf		ADRESH,W
		subwf	VBATH,W
		btfsc	STATUS,Z
		goto	CHARGE_COMPLETE_LOOP
		btfss	STATUS,C
		goto	INCREMENT_VBAT
		decf	VBATH,F
		movlw	VBAT405H
		subwf	VBATH,W
		btfss	STATUS,C
		goto	DELAY_LOOP
		goto	CHARGE_COMPLETE_LOOP		
INCREMENT_VBAT
		incf	VBATH,F
		goto	CHARGE_COMPLETE_LOOP

RETURN_TO_PRECONDITION
		bcf		BC_STATUS,CURRENT_RAMP
		bcf		BC_STATUS,REGULATE_CHARGE
		bsf		BC_STATUS,PRECONDITION
		movlw	PI_PWM1
		movwf	PWM1
		goto	VREF_PWM_LOOP

BATTERY_REMOVED
		bcf		GPIO,LED
		movlw	0x01
		andwf	BC_STATUS,F
		movlw	OV_DELAY
		movwf	DELAY_COUNT
		goto	DELAY_LOOP

TIMEOUT_FAULT
		btfss	BC_STATUS,PRECONDITION
		goto	CHARGE_COMPLETE_LOOP
		bsf		CCPR1L,7
		movlw	0x01
		andwf	BC_STATUS,F
TIMEOUT_FAULT_LOOP
		bsf		ADCON0,GO
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		btfsc	GPIO,LED
		goto	TURN_OFF_LED1
		bsf		GPIO,LED
		goto	CHECK_IF_BATTERY_REMOVED
TURN_OFF_LED1
		bcf		GPIO,LED
CHECK_IF_BATTERY_REMOVED
		movf	ADRESH,W
		movwf	VBATH
		sublw	VBAT20H
		btfsc	STATUS,C
		goto	CHARGE_CYCLE_START
		goto	TIMEOUT_FAULT_LOOP

TEMPERATURE_FAULT
		bcf		ADCON0,CHS0
		btfsc	GPIO,LED
		goto	TURN_OFF_LED2
		bsf		GPIO,LED
		goto	CHECK_IF_TEMPERATURE_OKAY
TURN_OFF_LED2
		bcf		GPIO,LED
CHECK_IF_TEMPERATURE_OKAY
		bsf		CCPR1L,7
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bsf		ADCON0,GO
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		movf	ADRESH,W
		movwf	TEMPH
		sublw	TEMP_HIGH_HYS
		btfss	STATUS,C
		goto	TEMPERATURE_FAULT
		movf	TEMPH,W
		sublw	TEMP_LOW_HYS
		btfsc	STATUS,C
		goto	TEMPERATURE_FAULT
		bsf		ADCON0,CHS0
		btfss	PIR1,TMR1IF
		goto	$-1
		bcf		PIR1,TMR1IF
		bcf		CCPR1L,7
		bsf		GPIO,LED
		goto	VREF_PWM_LOOP
		

;****************************************************************************
;PWM_TABLE
TABLE	CODE	0x0200
PWM_TABLE
		addwf	PCL,F
		nop							;0	
		nop							;1
		nop							;2
		nop							;3
		nop							;4
		nop							;5
		nop							;6	
		nop							;7
		nop							;8
		nop							;9
		nop							;10
		nop							;11
		nop							;12
		nop							;13
		nop							;14
		nop							;15
		nop							;16
		nop							;17
		nop							;18
		nop							;19
		nop							;20
		nop							;21
		nop							;22
		nop							;23
		nop							;24
		nop							;25
		nop							;26
		nop							;27
		nop							;28
		nop							;29
		nop							;30
		nop							;31
		nop							;32
		nop							;33
		nop							;34
		nop							;35
		nop							;36
		nop							;37
		nop							;38
		nop							;39
		nop							;40
		nop							;41
		nop							;42
		nop							;43
		nop							;44
		nop							;45
		nop							;46
		nop							;47
		nop							;48
		nop							;49
		nop							;50
		nop							;51
		nop							;52
		nop							;53
		nop							;54
		nop							;55
		nop							;56
		nop							;57
		nop							;58
		nop							;59
		nop							;60
		nop							;61
		nop							;62
		nop							;63
		nop							;64
		nop							;65
		nop							;66
		nop							;67
		nop							;68
		nop							;69
		nop							;70
		nop							;71
		nop							;72
		nop							;73
		nop							;74
		nop							;75
		nop							;76
		nop							;77
		nop							;78
		nop							;79
		nop							;80
		nop							;81
		nop							;82
		nop							;83
		nop							;84
		nop							;85
		nop							;86
		nop							;87
		nop							;88
		nop							;89
		nop							;90
		nop							;91
		nop							;92
		nop							;93
		nop							;94
		nop							;95
		nop							;96
		nop							;97
		nop							;98
		nop							;99
		nop							;100
		nop							;101
		nop							;102
		nop							;103
		nop							;104
		nop							;105
		nop							;106
		nop							;107
		nop							;108
		nop							;109
		nop							;110
		nop							;111
		nop							;112
		nop							;113
		nop							;114
		nop							;115
		nop							;116
		nop							;117
		nop							;118
		nop							;119
		nop							;120
		nop							;121
		nop							;122
		nop							;123
		nop							;124
		nop							;125
		nop							;126
		nop							;127
		nop							;128
		nop							;129
		nop							;130
		nop							;131
		nop							;132
		nop							;133
		nop							;134
		nop							;135
		nop							;136
		nop							;137
		nop							;138
		nop							;139
		nop							;140
		nop							;141
		nop							;142
		nop							;143
		nop							;144
		nop							;145
		nop							;146
		nop							;147
		nop							;148
		nop							;149
		nop							;150
		nop							;151
		nop							;152
		nop							;153
		nop							;154
		nop							;155
		nop							;156
		nop							;157
		nop							;158
		nop							;159
		nop							;160
		nop							;161
		nop							;162
		nop							;163
		nop							;164
		nop							;165
		nop							;166
		nop							;167
		nop							;168
		nop							;169
		nop							;170
		nop							;171
		nop							;172
		nop							;173
		nop							;174
		nop							;175
		nop							;176
		nop							;177
		nop							;178
		nop							;179
		nop							;180
		nop							;181
		nop							;182
		nop							;183
		nop							;184
		nop							;185
		nop							;186
		nop							;187
		nop							;188
		nop							;189
		nop							;190
		nop							;191
		nop							;192
		nop							;193
		nop							;194
		nop							;195
		nop							;196
		nop							;197
		nop							;198
		nop							;199
		nop							;200
		nop							;201
		nop							;202
		nop							;203
		nop							;204
		nop							;205
		nop							;206
		nop							;207
		nop							;208
		nop							;209
		nop							;210
		nop							;211
		nop							;212
		nop							;213
		nop							;214
		nop							;215
		nop							;216
		nop							;217
		nop							;218
		nop							;219
		nop							;220
		nop							;221
		nop							;222
		nop							;223
		nop							;224
		nop							;225
		nop							;226
		nop							;227
		nop							;228
		nop							;229
		nop							;230
		nop							;231
		nop							;232
		nop							;233
		nop							;234
		nop							;235
		nop							;236
		nop							;237
		nop							;238
		nop							;239
		nop							;240
		nop							;242
		nop							;243
		nop							;244
		nop							;245
		nop							;246
		nop							;247
		nop							;248
		nop							;249
		return						;250

; initialize eeprom locations

EE		CODE	0x2100
		DE	0x00, 0x00, 0x00, 0x00


		END                       ; directive 'end of program'

